searchNodes=[{"doc":"Bypass provides a quick way to create a custom plug that can be put in place instead of an actual HTTP server to return prebaked responses to client requests. This is most useful in tests, when you want to create a mock HTTP server and test how your HTTP client handles different types of responses from the server.Bypass supports Elixir 1.7 and OTP 20 and up. It works with Cowboy 1 and 2.UsageTo use Bypass in a test case, open a connection and use its port to connect your client to it.If you want to test what happens when the HTTP server goes down, use Bypass.down/1 to close the TCP socket and Bypass.up/1 to start listening on the same port again. Both functions block until the socket updates its state.Expect FunctionsYou can take any of the following approaches:expect/2 or expect_once/2 to install a generic function that all calls to bypass will useexpect/4 and/or expect_once/4 to install specific routes (method and path)stub/4 to install specific routes without expectationsa combination of the above, where the routes will be used first, and then the generic version will be used as defaultExampleIn the following example TwitterClient.start_link() takes the endpoint URL as its argument allowing us to make sure it will connect to the running instance of Bypass.defmodule TwitterClientTest do use ExUnit.Case, async: true setup do bypass = Bypass.open() {:ok, bypass: bypass} end test &quot;client can handle an error response&quot;, %{bypass: bypass} do Bypass.expect_once(bypass, &quot;POST&quot;, &quot;/1.1/statuses/update.json&quot;, fn conn -&gt; Plug.Conn.resp(conn, 429, ~s&lt;{&quot;errors&quot;: [{&quot;code&quot;: 88, &quot;message&quot;: &quot;Rate limit exceeded&quot;}]}&gt;) end) {:ok, client} = TwitterClient.start_link(url: endpoint_url(bypass.port)) assert {:error, :rate_limited} == TwitterClient.post_tweet(client, &quot;Elixir is awesome!&quot;) end test &quot;client can recover from server downtime&quot;, %{bypass: bypass} do Bypass.expect(bypass, fn conn -&gt; # We don&#39;t care about `request_path` or `method` for this test. Plug.Conn.resp(conn, 200, &quot;&quot;) end) {:ok, client} = TwitterClient.start_link(url: endpoint_url(bypass.port)) assert :ok == TwitterClient.post_tweet(client, &quot;Elixir is awesome!&quot;) # Blocks until the TCP socket is closed. Bypass.down(bypass) assert {:error, :noconnect} == TwitterClient.post_tweet(client, &quot;Elixir is awesome!&quot;) Bypass.up(bypass) # When testing a real client that is using e.g. https://github.com/fishcakez/connection # with https://github.com/ferd/backoff to handle reconnecting, we&#39;d have to loop for # a while until the client has reconnected. assert :ok == TwitterClient.post_tweet(client, &quot;Elixir is awesome!&quot;) end defp endpoint_url(port), do: &quot;http://localhost:\#{port}/&quot; endThat's all you need to do. Bypass automatically sets up an on_exit hook to close its socket when the test finishes running.Multiple concurrent Bypass instances are supported, all will have a different unique port. Concurrent requests are also supported on the same instance.Note: Bypass.open/0 must not be called in a setup_all blocks due to the way Bypass verifies the expectations at the end of each test.How to use with ESpecWhile Bypass primarily targets ExUnit, the official Elixir builtin test framework, it can also be used with ESpec. The test configuration is basically the same, there are only two differences:In your Mix config file, you must declare which test framework Bypass is being used with (defaults to :ex_unit). This simply disables the automatic integration with some hooks provided by ExUnit.config :bypass, test_framework: :especIn your specs, you must explicitly verify the declared expectations. You can do it in the finally block.defmodule TwitterClientSpec do use ESpec, async: true before do bypass = Bypass.open() {:shared, bypass: bypass} end finally do Bypass.verify_expectations!(shared.bypass) end specify &quot;the client can handle an error response&quot; do Bypass.expect_once(shared.bypass, &quot;POST&quot;, &quot;/1.1/statuses/update.json&quot;, fn conn -&gt; Plug.Conn.resp(conn, 429, ~s&lt;{&quot;errors&quot;: [{&quot;code&quot;: 88, &quot;message&quot;: &quot;Rate limit exceeded&quot;}]}&gt;) end) {:ok, client} = TwitterClient.start_link(url: endpoint_url(shared.bypass.port)) assert {:error, :rate_limited} == TwitterClient.post_tweet(client, &quot;Elixir is awesome!&quot;) end defp endpoint_url(port), do: &quot;http://localhost:\#{port}/&quot; endConfiguration optionsSet :enable_debug_log to true in the application environment to make Bypass log what it's doing:config :bypass, enable_debug_log: true","ref":"Bypass.html","title":"Bypass","type":"module"},{"doc":"Closes the TCP socket. Blocks until the operation is complete.Bypass.down(bypass)","ref":"Bypass.html#down/1","title":"Bypass.down/1","type":"function"},{"doc":"Expects the passed function to be called at least once regardless of the route.Bypass.expect(bypass, fn conn -&gt; assert &quot;/1.1/statuses/update.json&quot; == conn.request_path assert &quot;POST&quot; == conn.method Plug.Conn.resp(conn, 429, ~s&lt;{&quot;errors&quot;: [{&quot;code&quot;: 88, &quot;message&quot;: &quot;Rate limit exceeded&quot;}]}&gt;) end)","ref":"Bypass.html#expect/2","title":"Bypass.expect/2","type":"function"},{"doc":"Expects the passed function to be called at least once for the specified route (method and path).method is one of [&quot;GET&quot;, &quot;POST&quot;, &quot;HEAD&quot;, &quot;PUT&quot;, &quot;PATCH&quot;, &quot;DELETE&quot;, &quot;OPTIONS&quot;, &quot;CONNECT&quot;]path is the endpoint.Bypass.expect(bypass, &quot;POST&quot;, &quot;/1.1/statuses/update.json&quot;, fn conn -&gt; Agent.get_and_update(AgentModule, fn step_no -&gt; {step_no, step_no + 1} end) Plug.Conn.resp(conn, 429, ~s&lt;{&quot;errors&quot;: [{&quot;code&quot;: 88, &quot;message&quot;: &quot;Rate limit exceeded&quot;}]}&gt;) end)","ref":"Bypass.html#expect/4","title":"Bypass.expect/4","type":"function"},{"doc":"Expects the passed function to be called exactly once regardless of the route.Bypass.expect_once(bypass, fn conn -&gt; assert &quot;/1.1/statuses/update.json&quot; == conn.request_path assert &quot;POST&quot; == conn.method Plug.Conn.resp(conn, 429, ~s&lt;{&quot;errors&quot;: [{&quot;code&quot;: 88, &quot;message&quot;: &quot;Rate limit exceeded&quot;}]}&gt;) end)","ref":"Bypass.html#expect_once/2","title":"Bypass.expect_once/2","type":"function"},{"doc":"Expects the passed function to be called exactly once for the specified route (method and path).method is one of [&quot;GET&quot;, &quot;POST&quot;, &quot;HEAD&quot;, &quot;PUT&quot;, &quot;PATCH&quot;, &quot;DELETE&quot;, &quot;OPTIONS&quot;, &quot;CONNECT&quot;]path is the endpoint.Bypass.expect_once(bypass, &quot;POST&quot;, &quot;/1.1/statuses/update.json&quot;, fn conn -&gt; Agent.get_and_update(AgentModule, fn step_no -&gt; {step_no, step_no + 1} end) Plug.Conn.resp(conn, 429, ~s&lt;{&quot;errors&quot;: [{&quot;code&quot;: 88, &quot;message&quot;: &quot;Rate limit exceeded&quot;}]}&gt;) end)","ref":"Bypass.html#expect_once/4","title":"Bypass.expect_once/4","type":"function"},{"doc":"Starts an Elixir process running a minimal Plug app. The process is a HTTP handler and listens to requests on a TCP port on localhost.Use the other functions in this module to declare which requests are handled and set expectations on the calls.Optionsport - Optional TCP port to listen to requests.Examplesbypass = Bypass.open()Assign a specific port to a Bypass instance to listen on:bypass = Bypass.open(port: 1234)","ref":"Bypass.html#open/1","title":"Bypass.open/1","type":"function"},{"doc":"Makes a expection to pass.Bypass.expect(bypass, fn _conn -&gt; Bypass.pass(bypass) assert false end)","ref":"Bypass.html#pass/1","title":"Bypass.pass/1","type":"function"},{"doc":"Allows the function to be invoked zero or many times for the specified route (method and path).method is one of [&quot;GET&quot;, &quot;POST&quot;, &quot;HEAD&quot;, &quot;PUT&quot;, &quot;PATCH&quot;, &quot;DELETE&quot;, &quot;OPTIONS&quot;, &quot;CONNECT&quot;]path is the endpoint.Bypass.stub(bypass, &quot;POST&quot;, &quot;/1.1/statuses/update.json&quot;, fn conn -&gt; Agent.get_and_update(AgentModule, fn step_no -&gt; {step_no, step_no + 1} end) Plug.Conn.resp(conn, 429, ~s&lt;{&quot;errors&quot;: [{&quot;code&quot;: 88, &quot;message&quot;: &quot;Rate limit exceeded&quot;}]}&gt;) end)","ref":"Bypass.html#stub/4","title":"Bypass.stub/4","type":"function"},{"doc":"Re-opens the TCP socket on the same port. Blocks until the operation is complete.Bypass.up(bypass)","ref":"Bypass.html#up/1","title":"Bypass.up/1","type":"function"},{"doc":"Can be called to immediately verify if the declared request expectations have been met.Returns :ok on success and raises an error on failure.","ref":"Bypass.html#verify_expectations!/1","title":"Bypass.verify_expectations!/1","type":"function"},{"doc":"Represents a Bypass server process.","ref":"Bypass.html#t:t/0","title":"Bypass.t/0","type":"type"}]